@@ -51,7 +51,7 @@ LLVMPY_GetProcessTriple(const char **Out) {
  * Features are prefixed with '+' or '-' for enabled or disabled, respectively.
  * Features are separated by ','.
  */
-API_EXPORT(void)
+API_EXPORT(int)
 LLVMPY_GetHostCPUFeatures(const char **Out){
     llvm::StringMap<bool> features;
     std::ostringstream buf;
@@ -62,8 +62,10 @@ LLVMPY_GetHostCPUFeatures(const char **Out){
             }
             buf << ((F.second? "+": "-") + F.first()).str();
         }
+        *Out = LLVMPY_CreateString(buf.str().c_str());
+        return 1;
     }
-    *Out = LLVMPY_CreateString(buf.str().c_str());
+    return 0;
 }
 
 API_EXPORT(void)
@@ -88,13 +90,6 @@ LLVMPY_CreateTargetData(const char *StringRep)
     return LLVMCreateTargetData(StringRep);
 }
 
-API_EXPORT(void)
-LLVMPY_AddTargetData(LLVMTargetDataRef TD,
-                     LLVMPassManagerRef PM)
-{
-    LLVMAddTargetData(TD, PM);
-}
-
 
 //// Nothing is creating a TargetLibraryInfo
 //    void
@@ -129,7 +124,7 @@ LLVMPY_ABISizeOfElementType(LLVMTargetDataRef TD, LLVMTypeRef Ty)
     llvm::Type *tp = llvm::unwrap(Ty);
     if (!tp->isPointerTy())
         return -1;
-    tp = tp->getSequentialElementType();
+    tp = tp->getPointerElementType();
     return (long long) LLVMABISizeOfType(TD, llvm::wrap(tp));
 }
 
@@ -139,7 +134,7 @@ LLVMPY_ABIAlignmentOfElementType(LLVMTargetDataRef TD, LLVMTypeRef Ty)
     llvm::Type *tp = llvm::unwrap(Ty);
     if (!tp->isPointerTy())
         return -1;
-    tp = tp->getSequentialElementType();
+    tp = tp->getPointerElementType();
     return (long long) LLVMABIAlignmentOfType(TD, llvm::wrap(tp));
 }
 
@@ -199,9 +194,7 @@ LLVMPY_CreateTargetMachine(LLVMTargetRef T,
 
     CodeModel::Model cm;
     std::string cms(CodeModel);
-    if (cms == "jitdefault")
-        cm = CodeModel::JITDefault;
-    else if (cms == "small")
+    if (cms == "small")
         cm = CodeModel::Small;
     else if (cms == "kernel")
         cm = CodeModel::Kernel;
@@ -209,10 +202,18 @@ LLVMPY_CreateTargetMachine(LLVMTargetRef T,
         cm = CodeModel::Medium;
     else if (cms == "large")
         cm = CodeModel::Large;
-    else
-        cm = CodeModel::Default;
+    else if (cms == "default") // As per LLVM 5, needed for AOT
+        cm = CodeModel::Small;
+    else { // catches "jitdefault" and not set, as per LLVM 5, needed for MCJIT
+        // fall through, use model based on bitness
+        int bits = sizeof(void *);
+        if (bits == 4)
+            cm = CodeModel::Small;
+        else
+            cm = CodeModel::Large;
+    }
 
-    Reloc::Model rm;
+    Optional<Reloc::Model> rm;
     std::string rms(RelocModel);
     if (rms == "static")
         rm = Reloc::Static;
@@ -220,8 +221,6 @@ LLVMPY_CreateTargetMachine(LLVMTargetRef T,
         rm = Reloc::PIC_;
     else if (rms == "dynamicnopic")
         rm = Reloc::DynamicNoPIC;
-    else
-        rm = Reloc::Default;
 
     TargetOptions opt;
 //     opt.JITEmitDebugInfo = EmitJITDebug;
@@ -310,6 +309,17 @@ LLVMPY_DisposeMemoryBuffer(LLVMMemoryBufferRef MB)
     return LLVMDisposeMemoryBuffer(MB);
 }
 
+API_EXPORT(int)
+LLVMPY_HasSVMLSupport(void)
+{
+#ifdef HAVE_SVML
+    return 1;
+#else
+    return 0;
+#endif
+}
+
+
 /*
 
 If needed:
