@@ -52,13 +52,19 @@ def get_host_cpu_features():
     is available.  The returned value is an instance of ``FeatureMap`` class,
     which adds a new method ``.flatten()`` for returning a string suitable for
     use as the "features" argument to ``Target.create_target_machine()``.
+
+    If LLVM has not implemented this feature or it fails to get the information,
+    this function will raise a RuntimeError exception.
     """
     with ffi.OutputString() as out:
-        ffi.lib.LLVMPY_GetHostCPUFeatures(out)
         outdict = FeatureMap()
+        if not ffi.lib.LLVMPY_GetHostCPUFeatures(out):
+            raise RuntimeError("failed to get host cpu features.")
         flag_map = {'+': True, '-': False}
-        for feat in str(out).split(','):
-            outdict[feat[1:]] = flag_map[feat[0]]
+        content = str(out)
+        if content:  # protect against empty string
+            for feat in content.split(','):
+                outdict[feat[1:]] = flag_map[feat[0]]
         return outdict
 
 
@@ -144,18 +150,10 @@ class TargetData(ffi.ObjectRef):
             raise RuntimeError("Not a pointer type: %s" % (ty,))
         return size
 
-    def add_pass(self, pm):
-        """
-        Add a DataLayout pass to PassManager *pm*.
-        """
-        # AddTargetData claims ownership, so create a copy.
-        target_data = create_target_data(str(self))
-        ffi.lib.LLVMPY_AddTargetData(target_data, pm)
-
 
 RELOC = frozenset(['default', 'static', 'pic', 'dynamicnopic'])
-CODEMODEL = frozenset(['default', 'jitdefault', 'small', 'kernel',
-                       'medium', 'large'])
+CODEMODEL = frozenset(['default', 'jitdefault', 'small', 'kernel', 'medium',
+                       'large'])
 
 
 class Target(ffi.ObjectRef):
@@ -208,6 +206,10 @@ class Target(ffi.ObjectRef):
                               jitdebug=False, printmc=False):
         """
         Create a new TargetMachine for this target and the given options.
+
+        Specifying codemodel='default' will result in the use of the "small"
+        code model. Specifying codemodel='jitdefault' will result in the code
+        model being picked based on platform bitness (32="small", 64="large").
         """
         assert 0 <= opt <= 3
         assert reloc in RELOC
@@ -299,6 +301,15 @@ class TargetMachine(ffi.ObjectRef):
             ffi.lib.LLVMPY_GetTargetMachineTriple(self, out)
             return str(out)
 
+def has_svml():
+    """
+    Returns True if SVML was enabled at FFI support compile time.
+    """
+    if ffi.lib.LLVMPY_HasSVMLSupport() == 0:
+        return False
+    else:
+        return True
+
 
 # ============================================================================
 # FFI
@@ -306,6 +317,7 @@ class TargetMachine(ffi.ObjectRef):
 ffi.lib.LLVMPY_GetProcessTriple.argtypes = [POINTER(c_char_p)]
 
 ffi.lib.LLVMPY_GetHostCPUFeatures.argtypes = [POINTER(c_char_p)]
+ffi.lib.LLVMPY_GetHostCPUFeatures.restype = c_int
 
 ffi.lib.LLVMPY_GetDefaultTargetTriple.argtypes = [POINTER(c_char_p)]
 
@@ -326,9 +338,6 @@ ffi.lib.LLVMPY_DisposeTargetData.argtypes = [
     ffi.LLVMTargetDataRef,
 ]
 
-ffi.lib.LLVMPY_AddTargetData.argtypes = [ffi.LLVMTargetDataRef,
-                                         ffi.LLVMPassManagerRef]
-
 ffi.lib.LLVMPY_ABISizeOfType.argtypes = [ffi.LLVMTargetDataRef,
                                          ffi.LLVMTypeRef]
 ffi.lib.LLVMPY_ABISizeOfType.restype = c_longlong
@@ -400,3 +409,6 @@ ffi.lib.LLVMPY_CreateTargetMachineData.argtypes = [
     ffi.LLVMTargetMachineRef,
 ]
 ffi.lib.LLVMPY_CreateTargetMachineData.restype = ffi.LLVMTargetDataRef
+
+ffi.lib.LLVMPY_HasSVMLSupport.argtypes = []
+ffi.lib.LLVMPY_HasSVMLSupport.restype = c_int
